from enum import Enum

"""
统一枚举定义文件
"""


class ACLType(Enum):
    ACL_Unknown = "Unknown"
    ACL_Private = "private"
    ACL_Public_Read = "public-read"
    ACL_Public_Read_Write = "public-read-write"
    ACL_Authenticated_Read = "authenticated-read"
    ACL_Bucket_Owner_Read = "bucket-owner-read"
    ACL_Bucket_Owner_Full_Control = "bucket-owner-full-control"
    ACL_Bucket_Owner_Entrusted = "bucket-owner-entrusted"


def convert_acl_type(acl: str):
    for t in ACLType:
        if t.value == acl:
            return t
    return ACLType.ACL_Unknown


class StorageClassType(Enum):
    Storage_Unknown = "Unknown"
    # 标准存储
    Storage_Class_Standard = "STANDARD"
    # 低频访问存储
    Storage_Class_Ia = "IA"
    Storage_Class_INTELLIGENT_TIERING = "INTELLIGENT_TIERING"
    # 归档闪回存储
    Storage_Class_Archive_Fr = 'ARCHIVE_FR'
    # 冷归档存储
    Storage_Class_Cold_Archive = 'COLD_ARCHIVE'
    # 归档存储
    Storage_Class_Archive = 'ARCHIVE'


def convert_storage_class_type(storage_class: str):
    for t in StorageClassType:
        if t.value == storage_class:
            return t
    return StorageClassType.Storage_Unknown


class MetadataDirectiveType(Enum):
    Metadata_Directive_Unknown = "Unknown"
    Metadata_Directive_Copy = "COPY"
    Metadata_Directive_Replace = "REPLACE"


def convert_metadata_directive_type(s: str):
    for t in MetadataDirectiveType:
        if t.value == s:
            return t
    return MetadataDirectiveType.Metadata_Directive_Unknown


class AzRedundancyType(Enum):
    Az_Redundancy_Unknown = "Unknown"
    Az_Redundancy_Single_Az = "single-az"
    Az_Redundancy_Multi_Az = "multi-az"


def convert_az_redundancy_type(s: str):
    for t in AzRedundancyType:
        if t.value == s:
            return t
    return AzRedundancyType.Az_Redundancy_Unknown


class PermissionType(Enum):
    Permission_Unknown = "Unknown"
    Permission_Read = "READ"
    Permission_Write = "WRITE"
    Permission_Read_Acp = "READ_ACP"
    Permission_Write_Acp = "WRITE_ACP"
    Permission_Full_Control = "FULL_CONTROL"


def convert_permission_type(s: str):
    for t in PermissionType:
        if t.value == s:
            return t
    return PermissionType.Permission_Unknown


class GranteeType(Enum):
    Grantee_Unknown = "Unknown"
    Grantee_Group = "Group"
    Grantee_User = "CanonicalUser"


def convert_grantee_type(s: str):
    for t in GranteeType:
        if t.value == s:
            return t
    return GranteeType.Grantee_Unknown


class CannedType(Enum):
    Canned_Unknown = "Unknown"
    Canned_All_Users = "AllUsers"
    Canned_Authenticated_Users = "AuthenticatedUsers"


def convert_canned_type(s: str):
    for t in CannedType:
        if t.value == s:
            return t
    return CannedType.Canned_Unknown


class HttpMethodType(Enum):
    Http_Method_Get = "GET"
    Http_Method_Put = "PUT"
    Http_Method_Post = "POST"
    Http_Method_Delete = "DELETE"
    Http_Method_Head = "HEAD"


class DataTransferType(Enum):
    Data_Transfer_Init = 0
    Data_Transfer_Started = 1
    Data_Transfer_RW = 2
    Data_Transfer_Succeed = 3
    Data_Transfer_Failed = 4


class UploadEventType(Enum):
    Upload_Event_Create_Multipart_Upload_Succeed = 1
    Upload_Event_Create_Multipart_Upload_Failed = 2
    Upload_Event_Upload_Part_Succeed = 3
    Upload_Event_Upload_Part_Failed = 4
    Upload_Event_UploadPart_Aborted = 5
    Upload_Event_Complete_Multipart_Upload_Succeed = 6
    Upload_Event_Complete_Multipart_Upload_Failed = 7


class DownloadEventType(Enum):
    Download_Event_Create_TempFile_Succeed = 1
    Download_Event_Create_Temp_File_Failed = 2
    Download_Event_Download_Part_Succeed = 3
    Download_Event_Download_Part_Failed = 4
    Download_Event_Download_Part_Aborted = 5
    Download_Event_Rename_Temp_File_Succeed = 6
    Download_Event_Rename_Temp_File_Failed = 7


class RedirectType(Enum):
    Unknown = "Unknown"
    Mirror = "Mirror"
    Async = "Async"


def convert_redirect_type(s: str):
    for t in RedirectType:
        if t.value == s:
            return t
    return RedirectType.Unknown


class StatusType(Enum):
    Status_Unknown = "Unknown"
    Status_Enable = 'Enabled'
    Status_Disable = 'Disabled'


def convert_status_type(s: str):
    for t in StatusType:
        if t.value == s:
            return t
    return StatusType.Status_Unknown


class StorageClassInheritDirectiveType(Enum):
    Storage_Class_Unknown = "Unknown"
    Storage_Class_ID_Destination_Bucket = 'DESTINATION_BUCKET'
    Storage_Class_ID_Source_Object = 'SOURCE_OBJECT'


def convert_storage_class_inherit_directive_type(s: str):
    for t in StorageClassInheritDirectiveType:
        if t.value == s:
            return t
    return StorageClassInheritDirectiveType.Storage_Class_Unknown


class VersioningStatusType(Enum):
    Versioning_Unknown = "Unknown"
    Versioning_Status_Enabled = 'Enabled'
    Versioning_Status_Suspended = 'Suspended'


def convert_versioning_status_type(s: str):
    for t in VersioningStatusType:
        if t.value == s:
            return t
    return VersioningStatusType.Versioning_Unknown


class ProtocolType(Enum):
    Protocol_Unknown = "Unknown"
    Protocol_Http = 'http'
    Protocol_Https = 'https'


def convert_protocol_type(s: str):
    for t in ProtocolType:
        if t.value == s:
            return t
    return ProtocolType.Protocol_Unknown


class CertStatus(Enum):
    Cert_Unknown = "Unknown"
    Cert_Status_Bound = 'CertBound'
    Cert_Status_Unbound = 'CertUnbound'
    Cert_Status_Expired = 'CertExpired'


def convert_cert_status(s: str):
    for t in CertStatus:
        if t.value == s:
            return t
    return CertStatus.Cert_Unknown


class CopyEventType(Enum):
    Copy_Event_Create_Multipart_Upload_Succeed = 1
    Copy_Event_Create_Multipart_Upload_Failed = 2
    Copy_Event_Create_Part_Copy_Succeed = 3
    Copy_Event_Create_Part_Copy_Failed = 4
    Copy_Event_Create_Part_Copy_Aborted = 5
    Copy_Event_Completed_Multipart_Upload_Succeed = 6
    Copy_Event_Completed_Multipart_Upload_Failed = 7


class TierType(Enum):
    Tier_Unknown = "Unknown"
    Tier_Standard = "Standard"
    Tier_Expedited = "Expedited"
    Tier_Bulk = "Bulk"


def convert_tier_type(s: str):
    for t in TierType:
        if t.value == s:
            return t
    return TierType.Tier_Unknown



# -*- coding: utf-8 -*-
import os
import unittest

from tests.common import TosTestBase, random_bytes
from tos.exceptions import TosServerError
from tos.models2 import UploadedPart


class TestEncryption(TosTestBase):
    def test_customer_encryption(self):
        bucket_name = self.bucket_name + '-put-object-with-customer-encryption'
        key = self.random_key('.js')

        dis_bucket = self.bucket_name + "-put-dist-bucket-name"
        file_name = self.random_filename()
        content = random_bytes(1024 * 1024 * 4)
        self.client.create_bucket(bucket_name)
        self.client.create_bucket(dis_bucket)
        self.bucket_delete.append(bucket_name)
        self.bucket_delete.append(dis_bucket)

        self.client.put_object(bucket_name, key, content=content, ssec_algorithm=self.sseAlg,
                               ssec_key=self.sseKey, ssec_key_md5=self.sseKeyMd5)

        with self.assertRaises(TosServerError):
            self.client.get_object(bucket_name, key)

        out = self.client.get_object(bucket_name, key, ssec_algorithm=self.sseAlg,
                                     ssec_key=self.sseKey, ssec_key_md5=self.sseKeyMd5)
        self.assertEqual(out.read(), content)

        with self.assertRaises(TosServerError):
            self.client.get_object_to_file(bucket_name, key, file_name)
        self.client.get_object_to_file(bucket_name, key, file_name, ssec_algorithm=self.sseAlg, ssec_key=self.sseKey,
                                       ssec_key_md5=self.sseKeyMd5)
        self.assertFileContent(file_name, content)

        self.client.head_object(bucket_name, key, ssec_algorithm=self.sseAlg, ssec_key=self.sseKey,
                                ssec_key_md5=self.sseKeyMd5)
        with self.assertRaises(TosServerError):
            self.client.head_object(bucket_name, key)

        with self.assertRaises(TosServerError):
            self.client.copy_object(dis_bucket, key, bucket_name, key)
        self.client.copy_object(dis_bucket, key, bucket_name, key, copy_source_ssec_algorithm=self.sseAlg,
                                copy_source_ssec_key=self.sseKey, copy_source_ssec_key_md5=self.sseKeyMd5)
        self.client.head_object(dis_bucket, key)
        self.assertObjectContent(dis_bucket, key, content)

    def test_file_operator(self):
        bucket_name = self.bucket_name + '-encryption-file'
        key = self.random_key('.js')
        self.client.create_bucket(bucket_name)
        file_name = self.random_filename()
        file_download = self.random_filename()
        content = random_bytes(1024 * 1024 * 6)
        with open(file_name, 'wb') as f:
            f.write(content)
        self.client.put_object_from_file(bucket_name, key, file_name, ssec_algorithm=self.sseAlg, ssec_key=self.sseKey,
                                         ssec_key_md5=self.sseKeyMd5)
        with self.assertRaises(TosServerError):
            self.client.head_object(bucket_name, key)
        self.client.head_object(bucket_name, key, ssec_algorithm=self.sseAlg, ssec_key=self.sseKey,
                                ssec_key_md5=self.sseKeyMd5)
        out = self.client.get_object_to_file(bucket_name, key, file_download, ssec_algorithm=self.sseAlg,
                                             ssec_key=self.sseKey,
                                             ssec_key_md5=self.sseKeyMd5)

        self.assertFileContent(file_download, content)

    def test_server_encryption(self):
        bucket_name = self.bucket_name + '-put-object-with-customer-encryption'
        key = self.random_key('.js')

        content = random_bytes(1024 * 1024 * 4)
        self.client.create_bucket(bucket_name)
        self.bucket_delete.append(bucket_name)

        self.client.put_object(bucket_name, key, content=content, server_side_encryption=self.sseAlg)
        out = self.client.get_object(bucket_name, key)

    def test_upload_download_with_encryption(self):
        bucket_name = self.bucket_name + '-upload-file-with-customer-encryption'
        key = self.random_key('.js')
        file_name = self.random_filename()
        file_download = self.random_filename()
        self.bucket_delete.append(bucket_name)
        self.client.create_bucket(bucket_name)
        content = random_bytes(1024 * 1024 * 6)
        with open(file_name, 'wb') as f:
            f.write(content)

        self.client.upload_file(bucket_name, key, file_name, ssec_algorithm=self.sseAlg, ssec_key=self.sseKey,
                                ssec_key_md5=self.sseKeyMd5)
        with self.assertRaises(TosServerError):
            self.client.download_file(bucket_name, key, file_download)
        self.client.download_file(bucket_name, key, file_download, ssec_key=self.sseKey, ssec_algorithm=self.sseAlg,
                                  ssec_key_md5=self.sseKeyMd5)
        self.assertFileContent(file_download, content)

        with self.assertRaises(TosServerError):
            self.client.download_file(bucket_name, key, file_download)

    def test_multi_part(self):
        bucket_name = self.bucket_name + '-encryption'
        self.client.create_bucket(bucket_name)
        self.bucket_delete.append(bucket_name)
        key = self.random_key('.js')
        out = self.client.create_multipart_upload(bucket_name, key, ssec_key=self.sseKey, ssec_algorithm=self.sseAlg,
                                                  ssec_key_md5=self.sseKeyMd5)
        part_info = self.client.upload_part(bucket_name, key, out.upload_id, 1, content=b'123', ssec_key=self.sseKey,
                                            ssec_algorithm=self.sseAlg, ssec_key_md5=self.sseKeyMd5)
        parts = []
        parts.append(UploadedPart(part_number=1, etag=part_info.etag))
        self.client.complete_multipart_upload(bucket_name, key, out.upload_id, parts=parts)

    def test_copy_object(self):
        bucket_name = self.bucket_name + '-copy-encryption'
        src_key = self.random_key('.js')
        key = self.random_key('.js')
        content = random_bytes(1024 * 1025)
        self.client.create_bucket(bucket_name)
        self.bucket_delete.append(bucket_name)
        self.client.put_object(bucket_name, src_key, content=content, ssec_algorithm=self.sseAlg, ssec_key=self.sseKey,
                               ssec_key_md5=self.sseKeyMd5)
        self.client.copy_object(bucket_name, key, bucket_name, src_key, copy_source_ssec_algorithm=self.sseAlg,
                                copy_source_ssec_key=self.sseKey, copy_source_ssec_key_md5=self.sseKeyMd5,
                                ssec_algorithm=self.sseAlg, ssec_key=self.sseKey, ssec_key_md5=self.sseKeyMd5)
        with self.assertRaises(TosServerError):
            self.client.head_object(bucket_name, key)
        self.client.head_object(bucket_name, key, ssec_key=self.sseKey, ssec_algorithm=self.sseAlg,
                                ssec_key_md5=self.sseKeyMd5)
        out = self.client.get_object(bucket_name, key, ssec_key=self.sseKey, ssec_algorithm=self.sseAlg,
                                     ssec_key_md5=self.sseKeyMd5)
        self.assertEqual(out.read(), content)

    def test_copy_part(self):
        bucket_name = self.bucket_name + '-upload-part-copy'
        src_key = self.random_key('.js')
        key = self.random_key('.js')
        self.bucket_delete.append(bucket_name)
        self.client.create_bucket(bucket_name)
        content = random_bytes(1024 * 1024 * 6)
        self.client.put_object(bucket_name, src_key, content=content, ssec_algorithm=self.sseAlg, ssec_key=self.sseKey,
                               ssec_key_md5=self.sseKeyMd5)
        out = self.client.create_multipart_upload(bucket_name, key, ssec_key=self.sseKey, ssec_algorithm=self.sseAlg,
                                                  ssec_key_md5=self.sseKeyMd5)

        with self.assertRaises(TosServerError):
            self.client.upload_part_copy(out.bucket, out.key, out.upload_id, 1, bucket_name, src_key,
                                         copy_source_ssec_algorithm=self.sseAlg,
                                         copy_source_ssec_key=self.sseKey, copy_source_ssec_key_md5=self.sseKeyMd5)

        part = self.client.upload_part_copy(out.bucket, out.key, out.upload_id, 1, bucket_name, src_key,
                                            copy_source_ssec_algorithm=self.sseAlg,
                                            copy_source_ssec_key=self.sseKey, copy_source_ssec_key_md5=self.sseKeyMd5,
                                            ssec_algorithm=self.sseAlg, ssec_key=self.sseKey,
                                            ssec_key_md5=self.sseKeyMd5)
        part_2 = self.client.upload_part_copy(out.bucket, out.key, out.upload_id, 2, bucket_name, src_key,
                                              copy_source_ssec_algorithm=self.sseAlg,
                                              copy_source_ssec_key=self.sseKey, copy_source_ssec_key_md5=self.sseKeyMd5,
                                              ssec_algorithm=self.sseAlg, ssec_key=self.sseKey,
                                              ssec_key_md5=self.sseKeyMd5)
        self.client.complete_multipart_upload(bucket_name, key, out.upload_id, [part, part_2])
        with self.assertRaises(TosServerError):
            self.client.head_object(bucket_name, key)

        get_out = self.client.get_object(bucket_name, key, ssec_algorithm=self.sseAlg, ssec_key=self.sseKey,
                                         ssec_key_md5=self.sseKeyMd5)

        self.assertEqual(get_out.read(), content + content)

    def test_resumable_copy_object(self):
        bucket_name = self.bucket_name + '-upload-part-copy'
        src_key = self.random_key('.js')
        key = self.random_key('.js')
        path = '/tmp'
        self.bucket_delete.append(bucket_name)
        self.client.create_bucket(bucket_name)
        content = random_bytes(1024 * 1024 * 6)
        self.client.put_object(bucket_name, src_key, content=content, ssec_algorithm=self.sseAlg, ssec_key=self.sseKey,
                               ssec_key_md5=self.sseKeyMd5)

        self.client.resumable_copy_object(bucket_name, key, src_bucket=bucket_name, src_key=src_key,
                                          copy_source_ssec_algorithm=self.sseAlg, copy_source_ssec_key=self.sseKey,
                                          copy_source_ssec_key_md5=self.sseKeyMd5, checkpoint_file=path)

        self.client.head_object(bucket_name, key)
        self.client.resumable_copy_object(bucket_name, key, src_bucket=bucket_name, src_key=src_key,
                                          copy_source_ssec_algorithm=self.sseAlg, copy_source_ssec_key=self.sseKey,
                                          copy_source_ssec_key_md5=self.sseKeyMd5, checkpoint_file=path,
                                          ssec_algorithm=self.sseAlg, ssec_key_md5=self.sseKeyMd5, ssec_key=self.sseKey)
        with self.assertRaises(TosServerError):
            self.client.head_object(bucket_name, key)

        self.client.head_object(bucket_name, key, ssec_key=self.sseKey, ssec_algorithm=self.sseAlg,
                                ssec_key_md5=self.sseKeyMd5)


if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-

#: 连接超时时间
CONNECT_TIMEOUT = 60

GMT_DATE_FORMAT = '%a, %d %b %Y %H:%M:%S GMT'
DATE_FORMAT = '%Y%m%dT%H%M%SZ'
LAST_MODIFY_TIME_DATE_FORMAT = '%Y-%m-%dT%H:%M:%S.%fZ'
UNSIGNED_PAYLOAD = 'UNSIGNED-PAYLOAD'
ECS_DATE_FORMAT = '%Y-%m-%dT%H:%M:%S%z'

CHUNK_SIZE = 8 * 1024
PAYLOAD_BUFFER = 1024 * 1024
EMPTY_SHA256_HASH = 'e3b0c44298fc1c149afbf4c8996fb92427ae41e4649b934ca495991b7852b855'

DEFAULT_MIMETYPE = 'binary/octet-stream'
SLEEP_BASE_TIME = 0.1

WHITE_LIST_FUNCTION = ['create_bucket', 'delete_bucket', 'create_multipart_upload', 'complete_multipart_upload',
                       'abort_multipart_upload', 'set_object_meta', 'put_object_acl', 'delete_object', 'put_object',
                       'upload_part', 'put_bucket_cors', 'delete_bucket_cors', 'put_bucket_storage_class',
                       'put_bucket_lifecycle', 'delete_bucket_lifecycle', 'put_bucket_policy', 'delete_bucket_policy',
                       'put_bucket_mirror_back', 'delete_bucket_mirror_back', 'put_object_tagging',
                       'delete_object_tagging', 'put_bucket_acl', 'put_fetch_task', 'put_bucket_replication',
                       'put_bucket_versioning', 'put_bucket_website', 'delete_bucket_website',
                       'put_bucket_notification', 'put_bucket_custom_domain', 'delete_bucket_custom_domain',
                       'put_bucket_real_time_log', 'delete_bucket_real_time_log', 'restore_object', 'rename_object',
                       'put_bucket_rename', 'delete_bucket_rename', 'put_bucket_tagging', 'delete_bucket_tagging']

CALLBACK_FUNCTION = ['put_object', 'complete_multipart_upload']
CLIENT_ENCRYPTION_ALGORITHM = ['AES256']
SERVER_ENCRYPTION_ALGORITHM = ['AES256']

MAX_PART_NUMBER = 10000

MIN_PART_SIZE = 5242880
MAX_PART_SIZE = 5368709120

MIN_TRAFFIC_LIMIT = 819200
MAX_TRAFFIC_LIMIT = 838860800

SIGNATURE_QUERY_LOWER = "x-tos-signature"
V4_PREFIX = "x-tos"



import time

import tos
from tests.common import TosTestBase, random_bytes
from tos import VersioningStatusType
from tos.models2 import Tag, ObjectTobeDeleted


class TestWithVersion(TosTestBase):
    def test_get_version(self):
        bucket_name = self.bucket_name + '-with-version'
        dist_bucket = self.bucket_name + 'dist'
        self.bucket_delete.append(bucket_name)
        self.bucket_delete.append(dist_bucket)
        key = self.random_key('.js')
        self.client.create_bucket(bucket_name)
        self.client.create_bucket(dist_bucket)
        self.version_client.put_bucket_versioning(bucket_name, True)
        self.version_client.put_bucket_versioning(dist_bucket, True)
        time.sleep(60)
        out_v1 = self.client.put_object(bucket_name, key, content=b'123')
        self.assertIsNotNone(out_v1.version_id)
        self.assertObjectContent(bucket_name, key, content=b'123')

        out_v2 = self.client.put_object(bucket_name, key, content=b'456')
        self.assertIsNotNone(out_v2.version_id)
        self.assertObjectContent(bucket_name, key, content=b'456')

        tag_set = []
        tag_set.append(Tag(
            key='1',
            value='2'
        ))
        put_out = self.client.put_object_tagging(bucket=bucket_name, key=key, tag_set=tag_set,
                                                 version_id=out_v1.version_id)
        out = self.client.get_object_tagging(bucket_name, key, version_id=out_v1.version_id)
        self.assertEqual(out.version_id, out_v1.version_id)
        self.assertIsNotNone(out.tag_set[0].key, '1')
        self.assertIsNotNone(out.tag_set[0].value, '2')

        tag_set = []
        tag_set.append(Tag(
            key='2',
            value='3'
        ))
        put_out = self.client.put_object_tagging(bucket=bucket_name, key=key, tag_set=tag_set,
                                                 version_id=out_v2.version_id)
        out = self.client.get_object_tagging(bucket_name, key, version_id=out_v2.version_id)
        self.assertEqual(out.version_id, out_v2.version_id)
        self.assertIsNotNone(out.tag_set[0].key, '2')
        self.assertIsNotNone(out.tag_set[0].value, '3')

        self.assertEqual(self.client.get_object(bucket_name, key, version_id=out_v1.version_id).read(), b'123')
        self.assertEqual(self.client.get_object(bucket_name, key, version_id=out_v2.version_id).read(), b'456')

        out = self.client.copy_object(dist_bucket, key, bucket_name, key, src_version_id=out_v1.version_id)
        self.assertEqual(out.copy_source_version_id, out_v1.version_id)

        task = self.client.create_multipart_upload(bucket_name, 'testcopyversion')
        part = []
        out = self.client.upload_part_copy(bucket_name, task.key, task.upload_id, 1, bucket_name, key,
                                           src_version_id=out_v1.version_id)
        part.append(tos.models2.UploadedPart(1, out.etag))
        self.client.complete_multipart_upload(bucket_name, task.key, task.upload_id, part)
        copy_out = self.client.get_object(task.bucket, task.key)
        self.assertEqual(copy_out.read(), b'123')

        task = self.client.create_multipart_upload(bucket_name, 'testcopyversion2')
        part = []
        out = self.client.upload_part_copy(bucket_name, task.key, task.upload_id, 1, bucket_name, key,
                                           src_version_id=out_v2.version_id)
        part.append(tos.models2.UploadedPart(1, out.etag))
        self.client.complete_multipart_upload(bucket_name, task.key, task.upload_id, part)
        copy_out = self.client.get_object(task.bucket, task.key)
        self.assertEqual(copy_out.read(), b'456')

    def test_list_with_version(self):
        bucket_name = self.bucket_name + '-with-version-2'
        self.bucket_delete.append(bucket_name)
        key = self.random_key('.js')
        content = random_bytes(100)
        self.client.create_bucket(bucket_name)
        self.version_client.put_bucket_versioning(bucket_name, True)
        time.sleep(30)
        self.client.put_object(bucket_name, 'test.txt', content=content,
                               storage_class=tos.StorageClassType.Storage_Class_Ia)
        self.client.put_object(bucket_name, 'test.txt', content=content,
                               storage_class=tos.StorageClassType.Storage_Class_Ia)
        out = self.client.list_object_versions(bucket_name)
        for version in out.versions:
            self.assertIsNotNone(version.version_id)
            self.assertIsNotNone(version.etag)
            self.assertIsNotNone(version.owner)
            self.assertIsNotNone(version.is_latest)
            self.assertEqual(version.key, 'test.txt')
            self.assertEqual(version.storage_class, tos.StorageClassType.Storage_Class_Ia)

        self.client.put_object(bucket_name, 'test2.txt', content='123')
        out_1 = self.client.list_object_versions(bucket_name, max_keys=1)
        self.assertEqual(out_1.max_keys, 1)
        self.assertIsNotNone(out_1.next_version_id_marker)
        self.assertIsNotNone(out_1.next_key_marker)
        out_2 = self.client.list_object_versions(bucket_name, key_marker=out_1.next_key_marker,
                                                 version_id_marker=out_1.next_version_id_marker)

        self.client.put_object(bucket_name, 'func/')
        self.client.put_object(bucket_name, 'func2/')
        self.client.put_object(bucket_name, 'func3/')
        out = self.client.list_object_versions(bucket_name, delimiter='/')
        self.assertTrue(len(out.common_prefixes), 3)
        self.assertFalse(out_2.is_truncated)

    def test_download_copy_version(self):
        bucket_name = self.bucket_name + '-with-download-copy-version'
        dist_bucket = self.bucket_name + 'dist'
        file_name = self.random_filename()
        self.bucket_delete.append(bucket_name)
        self.bucket_delete.append(dist_bucket)
        content1 = random_bytes(1024)
        content2 = random_bytes(1024)
        key = self.random_key('.js')
        self.client.create_bucket(bucket_name)
        self.client.create_bucket(dist_bucket)
        self.version_client.put_bucket_versioning(bucket_name, True)
        self.version_client.put_bucket_versioning(dist_bucket, True)
        time.sleep(60)
        rsp = self.version_client.get_bucket_version(bucket_name)
        self.assertEqual(rsp.status, VersioningStatusType.Versioning_Status_Enabled)
        rsp = self.version_client.get_bucket_version(dist_bucket)
        self.assertEqual(rsp.status, VersioningStatusType.Versioning_Status_Enabled)
        out = self.client.put_object(bucket_name, key, content=content1)
        self.client.put_object(bucket_name, key, content=content2)
        self.client.download_file(bucket_name, key, file_name, version_id=out.version_id)
        self.assertFileContent(file_name, content1)

        self.client.resumable_copy_object(dist_bucket, key, bucket_name, key, src_version_id=out.version_id)
        self.assertObjectContent(dist_bucket, key, content1)

    def test_delete_version(self):
        bucket_name = self.bucket_name + '-with-download-copy-version'
        self.bucket_delete.append(bucket_name)
        self.client.create_bucket(bucket_name)
        self.version_client.put_bucket_versioning(bucket_name, True)
        time.sleep(10)
        delete = []
        v1 = self.client.put_object(bucket_name, '1')
        delete.append(ObjectTobeDeleted('1', v1.version_id))
        v2 = self.client.put_object(bucket_name, '1')
        delete.append(ObjectTobeDeleted('1', v2.version_id))
        out = self.client.delete_multi_objects(bucket_name, delete)
        self.assertTrue(len(out.deleted) == 2)

        v1 = self.client.put_object(bucket_name, '1')
        delete.append(ObjectTobeDeleted('1', v1.version_id))
        v2 = self.client.put_object(bucket_name, '1')
        delete.append(ObjectTobeDeleted('1', v2.version_id))
        out = self.client.delete_multi_objects(bucket_name, delete, quiet=True)
        self.assertTrue(len(out.deleted) == 0)
